@@ -1,7 +1,7 @@
 /*
  *  This file is part of gpw.
  *
- *  Copyright (C) 2013 Bruce Korb, all rights reserved.
+ *  Copyright (C) 2013-2015 Bruce Korb, all rights reserved.
  *  This is free software. It is licensed for use, modification and
  *  redistribution under the terms of the GNU General Public License,
  *  version 3 or later <http://gnu.org/licenses/gpl.html>
@@ -108,28 +108,39 @@ get_seed_text(void)
  * convert version to a number.  Limits version numbers to 4095.4095.4095
  * and ignores anything beyond the third component.
  */
-static void
+static uint32_t
 ver_str_to_number(void)
 {
     char const * pz = GNU_PW_MGR_VERSION;
-    int32_t     shift = 20;
+    int32_t   shift = 20;
+    uint32_t  res   = 0;
 
     errno = 0;
-    seed_version = 0;
 
     for (;;) {
-        char const * pn;
+        char * pn;
         uint32_t v = strtoul(pz, &pn, 10);
         if ((v >= (1<<10)) || (errno != 0))
-            break;
-        seed_version += v << shift;
-        if (*pn != '.')
-            break;
-        pz = pn + 1;
-        shift -= 10;
-        if (shift < 0)
-            break;
+            die(GNU_PW_MGR_EXIT_CODING_ERROR, bad_vers);
+        res += v << shift;
+        switch (*pn) {
+        case '-': case '\0':
+            goto return_res;
+
+        case '.':
+            shift -= 10;
+            if (shift < 0)
+                goto return_res;
+            pz = pn + 1;
+            continue;
+
+        default:
+            die(GNU_PW_MGR_EXIT_CODING_ERROR, bad_vers);
+        }
     }
+
+ return_res:
+    return res;
 }
 
 /**
@@ -140,7 +151,7 @@ static void
 add_seed(void)
 {
     char const * cfg_text = load_config_file();
-    ver_str_to_number();
+    uint32_t seed_ver = ver_str_to_number();
 
     {
         char * tag = scribble_get(sizeof (tag_fmt) + strlen(OPT_ARG(TAG)));
@@ -168,7 +179,7 @@ add_seed(void)
 
         seed_txt = get_seed_text();
 
-        fprintf(fp, cfg_fmt, OPT_ARG(TAG), seed_version, seed_txt);
+        fprintf(fp, cfg_fmt, OPT_ARG(TAG), seed_ver, seed_txt);
         if (p != NULL)
             fputs(p, fp);
         fclose(fp);
